/*----------------------------------------------------------------------------
 * Name:    LED.c
 * Purpose: low level LED functions
 * Note(s):
 *----------------------------------------------------------------------------
 * Modified by Antronics Ltd for use with the Arduino Due.
 * For details, see: http://www.element14.com/community/roadTestReviews/1453
 *                                                                 15 May 2013
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 20009-2011 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include "SAM3X.h"                        /* SAM3X definitions                */
#include "LED.h"

// This is a half-baked attempt at making a generic LED selector;
// The array elements give the pin masks, but the corresponding Ports are
// hard-coded!
// Also, the port setup was hard-coded!
// (and the Keil original didn't include the array dimension)
//                                        'TX' LED  'L' LED   'RX' LED
//                                        Port A    Port B    Port C
const unsigned long led_mask[LED_NUM] = { PIO_PA21, PIO_PB27, PIO_PC30 };

/*----------------------------------------------------------------------------
  initialize LED Pins
 *----------------------------------------------------------------------------*/
void LED_Init (void) {
  PMC->PMC_WPMR = 0x504D4300;             /* Disable write protect            */

  PMC->PMC_PCER0 = ((1UL << ID_PIOA) |    /* enable PIOA clock                */
                    (1UL << ID_PIOB) |    /* enable PIOB clock                */  
                    (1UL << ID_PIOC)  );  /* enable PIOC clock                */  

  PIOA->PIO_PER  =
  PIOA->PIO_OER  =
  PIOA->PIO_PUDR =
  PIOA->PIO_OWER = (led_mask[0]);         /* Setup PIO_PA12     for LED       */

#if LED_NUM > 1
  PIOB->PIO_PER  =
  PIOB->PIO_OER  =
  PIOB->PIO_PUDR =
  PIOB->PIO_OWER = (led_mask[1] ); 							/* Setup 1 LED on Port-B      */
	
#if LED_NUM == 3
  PIOC->PIO_PER  =
  PIOC->PIO_OER  =
  PIOC->PIO_PUDR =
  PIOC->PIO_OWER = (led_mask[2] ); 							/* Setup 1 LED on Port-B      */
	
#elif LED_NUM > 3
#error LED_NUM must be 2 or 3
#endif
#endif

  PMC->PMC_WPMR = 0x504D4301;             /* Enable write protect             */

  LED_Out (0);                            /* switch LEDs off                  */
}

/*----------------------------------------------------------------------------
  Function that turns on requested LED
 *----------------------------------------------------------------------------*/
void LED_On (unsigned int num) {

  if (num < LED_NUM) 
	{
		switch( num )
		{
			case 0: PIOA->PIO_CODR = led_mask[num]; break;
      case 1: PIOB->PIO_CODR = led_mask[num]; break;
      case 2: PIOC->PIO_CODR = led_mask[num]; break;
		}
  }
}

/*----------------------------------------------------------------------------
  Function that turns off requested LED
 *----------------------------------------------------------------------------*/
void LED_Off (unsigned int num) {

  if (num < LED_NUM) 
	{
		switch( num )
		{
			case 0: PIOA->PIO_SODR = led_mask[num]; break;
      case 1: PIOB->PIO_SODR = led_mask[num]; break;
      case 2: PIOC->PIO_SODR = led_mask[num]; break;
		}
  }
}

/*----------------------------------------------------------------------------
  Function that outputs value to LEDs
 *----------------------------------------------------------------------------*/
void LED_Out(unsigned int value) {
  int i;

  for (i = 0; i < LED_NUM; i++) {
    if (value & (1<<i)) {
      LED_On (i);
    } else {
      LED_Off(i);
    }
  }
}
